#include "ChannelController.h"
#include "StatusLed.h"
#include "cmsis_os2.h"
#include "FreeRTOS.h"
#include "task.h"
#include "MAX40080Driver.h"
#include "MAX5715.h"
#include "INA219Driver.h"
#include "SevenSegment.h"
#include "ButtonDriver.h"

#define MAX_ERROR_MSG_LEN 256

extern osMutexId_t ch1_ctl_statusHandle;
extern osMutexId_t ch2_ctl_statusHandle;

static osMutexId_t *chMutexes[NUMBER_OF_CHANNELS] = { &ch1_ctl_statusHandle, &ch2_ctl_statusHandle };
static int isRunning[NUMBER_OF_CHANNELS];
static int isError[NUMBER_OF_CHANNELS];
static float configuredVoltage[NUMBER_OF_CHANNELS];
static float actualVoltage[NUMBER_OF_CHANNELS];
static float actualCurrent[NUMBER_OF_CHANNELS];
static GPIO_TypeDef *chDisableGpio[NUMBER_OF_CHANNELS] = { GPIOF, GPIOE };
static uint16_t chDisableGpioPin[NUMBER_OF_CHANNELS] = { GPIO_PIN_13, GPIO_PIN_9 };

int ChannelController_IsRunning(int channelNo) {
	int output;
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	output = isRunning[channelNo];
	osMutexRelease(*(chMutexes[channelNo]));
	return output;
}

int ChannelController_IsError(int channelNo) {
	int output;
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	output = isError[channelNo];
	osMutexRelease(*(chMutexes[channelNo]));
	return output;
}

void ChannelController_Start(int channelNo) {
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	ChannelController_UpdateDacVoltage(channelNo);
	HAL_GPIO_WritePin(chDisableGpio[channelNo], chDisableGpioPin[channelNo], 0);
	isRunning[channelNo] = 1;
	StatusLed_SetState(channelNo, STATUS_LED_GREEN, 0);
	osMutexRelease(*(chMutexes[channelNo]));
}

void ChannelController_Stop(int channelNo) {
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	HAL_GPIO_WritePin(chDisableGpio[channelNo], chDisableGpioPin[channelNo], 1);
	ChannelController_SetDacToZero(channelNo);
	isRunning[channelNo] = 0;
	StatusLed_SetState(channelNo, STATUS_LED_OFF, 0);
	osMutexRelease(*(chMutexes[channelNo]));
}

void ChannelController_SetVoltage(int channelNo, float voltage) {
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	configuredVoltage[channelNo] = voltage;
	ChannelController_UpdateDacVoltage(channelNo);
	osMutexRelease(*(chMutexes[channelNo]));
}

float ChannelController_GetConfiguredVoltage(int channelNo) {
	float output;
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	output = configuredVoltage[channelNo];
	osMutexRelease(*(chMutexes[channelNo]));
	return output;
}

float ChannelController_GetActualCurrent(int channelNo) {
	float output;
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	output = actualCurrent[channelNo];
	osMutexRelease(*(chMutexes[channelNo]));
	return output;
}

void ChannelController_SetDacToValue(int channelNo, uint16_t value) {
	MAX5715_SetDAC(channelNo, value);
}

void ChannelController_SetDacToZero(int channelNo) {
	ChannelController_SetDacToValue(channelNo, 0);
}

void ChannelController_UpdateDacVoltage(int channelNo) {
#define RDIV_R3 33800.0f
#define RDIV_R2 12000.0f
#define RDIV_R1 81000.0f

	float value = RDIV_R3 / RDIV_R2 - RDIV_R3 / RDIV_R1 * (configuredVoltage[channelNo] - 1) + 1;
	int valueI = value * 1000;
	if (valueI < 0) {
		valueI = 0;
	}

	if (valueI > 4095) {
		valueI = 4095;
	}

	ChannelController_SetDacToValue(channelNo, valueI);
}

float ChannelController_GetActualVoltage(int channelNo) {
	float output;
	osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
	output = actualVoltage[channelNo];
	osMutexRelease(*(chMutexes[channelNo]));
	return output;
}

void ChannelController_CalibrateChannel(int channelNo) {
	SevenSegment_Clear();

	// waste previous press event
	ButtonDriver_WasPressed(channelNo);

	uint16_t digitA0 = DISPA_SEG_A | DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_F;
	uint16_t digitA1 = DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_G | DISPA_SEG_E | DISPA_SEG_F;
	uint16_t digitA2 = DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_F;
	uint16_t digitA3 = DISPA_SEG_E | DISPA_SEG_F;

	while (!ButtonDriver_WasPressed(channelNo)) {
		SevenSegment_SetDigit(DISPA_DIGIT_0_GREEN, digitA0);
		SevenSegment_SetDigit(DISPA_DIGIT_0_RED, digitA0);
		SevenSegment_SetDigit(DISPA_DIGIT_1_GREEN, digitA1);
		SevenSegment_SetDigit(DISPA_DIGIT_1_RED, digitA1);
		SevenSegment_SetDigit(DISPA_DIGIT_2_GREEN, digitA2);
		SevenSegment_SetDigit(DISPA_DIGIT_2_RED, digitA2);
		SevenSegment_SetDigit(DISPA_DIGIT_3_GREEN, digitA3);
		SevenSegment_SetDigit(DISPA_DIGIT_3_RED, digitA3);
		vTaskDelay(500);
		SevenSegment_Clear();
		vTaskDelay(500);
	}

	SevenSegment_SetDigit(DISPA_DIGIT_0_GREEN, digitA0);
	SevenSegment_SetDigit(DISPA_DIGIT_0_RED, digitA0);
	SevenSegment_SetDigit(DISPA_DIGIT_1_GREEN, digitA1);
	SevenSegment_SetDigit(DISPA_DIGIT_1_RED, digitA1);
	SevenSegment_SetDigit(DISPA_DIGIT_2_GREEN, digitA2);
	SevenSegment_SetDigit(DISPA_DIGIT_2_RED, digitA2);
	SevenSegment_SetDigit(DISPA_DIGIT_3_GREEN, digitA3);
	SevenSegment_SetDigit(DISPA_DIGIT_3_RED, digitA3);

	ChannelController_Start(channelNo);

	CalibrationRecord caliRec;
	for (int i = 0; i < 4096; i++) {
		TickType_t start = xTaskGetTickCount();
		ChannelController_SetDacToValue(channelNo, i);
		vTaskDelay(5);


		TickType_t timestamp;
		float current;
		float voltage;
		MAX40080Driver_GetLastSample(&timestamp, &current, &voltage);

		if (timestamp <= start) {
			__BKPT(0);
		}

		caliRec.realVoltage[i] = voltage;
		caliRec.isValid[i] = 1;

		if (i > 1024 * 3) {
			digitA0 |= DISPA_SEG_DOT;
			digitA1 |= DISPA_SEG_DOT;
			digitA2 |= DISPA_SEG_DOT;

			if (i & 128) {
				digitA3 |= DISPA_SEG_DOT;
			} else {
				digitA3 &= ~DISPA_SEG_DOT;
			}
		} else if (i > 1024 * 2) {
			digitA0 |= DISPA_SEG_DOT;
			digitA1 |= DISPA_SEG_DOT;
			if (i & 128) {
				digitA2 |= DISPA_SEG_DOT;
			} else {
				digitA2 &= ~DISPA_SEG_DOT;
			}
		} else if (i > 1024 * 1) {
			digitA0 |= DISPA_SEG_DOT;
			if (i & 128) {
				digitA1 |= DISPA_SEG_DOT;
			} else {
				digitA1 &= ~DISPA_SEG_DOT;
			}
		} else {
			if (i & 128) {
				digitA0 |= DISPA_SEG_DOT;
			} else {
				digitA0 &= ~DISPA_SEG_DOT;
			}
		}

		int circSegment = (i >> 6) % 6;
		uint16_t circularProgressBarSegments1[6] = { DISPB_SEG_A, DISPB_SEG_C, DISPB_SEG_D, DISPB_SEG_E, DISPB_SEG_G, DISPB_SEG_H };
		uint16_t circularProgressBarSegments2[6] = { DISPB_SEG_B, 0, 0, DISPB_SEG_F, 0, 0 };

		SevenSegment_SetDigit(DISPB_CYC1, circularProgressBarSegments1[circSegment]);
		SevenSegment_SetDigit(DISPB_CYC2, circularProgressBarSegments2[circSegment]);

		SevenSegment_SetDigit(DISPA_DIGIT_0_GREEN, digitA0);
		SevenSegment_SetDigit(DISPA_DIGIT_1_GREEN, digitA1);
		SevenSegment_SetDigit(DISPA_DIGIT_2_GREEN, digitA2);
		SevenSegment_SetDigit(DISPA_DIGIT_3_GREEN, digitA3);
	}

	SevenSegment_Clear();
	digitA0 &= ~DISPA_SEG_DOT;
	digitA1 &= ~DISPA_SEG_DOT;
	digitA2 &= ~DISPA_SEG_DOT;
	digitA3 &= ~DISPA_SEG_DOT;
	for (int i = 0; i < 3; i++) {
		SevenSegment_SetDigit(DISPA_DIGIT_0_GREEN, digitA0);
		SevenSegment_SetDigit(DISPA_DIGIT_1_GREEN, digitA1);
		SevenSegment_SetDigit(DISPA_DIGIT_2_GREEN, digitA2);
		SevenSegment_SetDigit(DISPA_DIGIT_3_GREEN, digitA3);
		vTaskDelay(500);
		SevenSegment_Clear();
		vTaskDelay(500);
	}

	SevenSegment_Clear();
}

void ChannelController_ThreadEntry(int channelNo) {
	while (1) {
		TickType_t now;
		TickType_t timestamp;
		float current;
		float voltage;

		if (channelNo == 0) {
			MAX40080Driver_GetLastSample(&timestamp, &current, &voltage);
		} else if (channelNo == 1) {
			INA219Driver_GetLastSample(&timestamp, &current, &voltage);
		} else {
			timestamp = 0;
			current = 0;
			voltage = 0;
		}
		now = xTaskGetTickCount();

		osMutexAcquire(*(chMutexes[channelNo]), portMAX_DELAY);
		actualCurrent[channelNo] = current;
		actualVoltage[channelNo] = voltage;

		if (now - timestamp > 100) {
			isError[channelNo] = 1;
			StatusLed_SetState(channelNo, STATUS_LED_RED, 1);
		} else {
			isError[channelNo] = 0;
			if (isRunning[channelNo]) {
				StatusLed_SetState(channelNo, STATUS_LED_GREEN, 0);
			} else {
				StatusLed_SetState(channelNo, STATUS_LED_OFF, 0);
			}
		}
		osMutexRelease(*(chMutexes[channelNo]));

		vTaskDelay(1);
	}
}
