#include <MAX40080Driver.h>
#include "MAX40080.h"
#include "stm32l5xx_hal.h"
#include "FreeRTOS.h"
#include "task.h"
#include "UART.h"
#include "cmsis_os2.h"

static MAX40080_Status MAX40080Driver_ReinitializeSensor() {
	MAX40080_Status status;

	// This function is called at two situations:
	// 1) at begining in time when sensor is configured first time
	// 2) when some unrecoverable error occur and for example I2C bus get stuck driven by someone.

	MAX40080_Deinit();

	// enable MAX40080 power
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_6, 0);

	// power cycle sensor by EN pin
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_6, 0);
	vTaskDelay(pdMS_TO_TICKS(1));
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_6, 1);
	vTaskDelay(pdMS_TO_TICKS(1));

	status = MAX40080_Init();
	if (status) {
		UART_PrintString("MAX40080 INIT Failed. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	MAX40080_Configuration config;
	MAX40080_GetDefaultConfiguration(&config);
	config.operatingMode = MAX40080_OperationMode_Active;
	config.adcSampleRate = MAX40080_AdcSampleRate_Both_at_0_5_ksps;
	config.inputRange = MAX40080_InputRange_50mV;
	config.digitalFilter = MAX40080_DigitalFilter_128_samples;

	status = MAX40080_SetConfiguration(&config);
	if (status) {
		UART_PrintString("Error while configuring MAX40080. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	MAX40080_FifoConfiguration fifoConfig;
	MAX40080_GetFifoDefaultConfiguration(&fifoConfig);
	fifoConfig.storingMode = MAX40080_FifoStoringMode_CurrentAndVoltage;
	fifoConfig.overflowWarningLevel = 1;

	status = MAX40080_SetFifoConfiguration(&fifoConfig);
	if (status) {
		UART_PrintString("Error while configuring MAX40080 FIFO. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	status = MAX40080_EnableInterrupts(MAX40080_Interrupt_FifoOverflowWarning | MAX40080_Interrupt_FifoOverflown);
	if (status) {
		UART_PrintString("Error while configuring MAX40080 Interrupts. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	status = MAX40080_FlushFifo();
	if (status) {
		UART_PrintString("Error while flushing MAX40080 FIFO. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	status = MAX40080_ClearPendingInterrupts(0xFF);
	if (status) {
		UART_PrintString("Error while clearing MAX40080 interrupt flags. Error code: ");
		UART_PrintFormattedInteger("%d", status);
		UART_PrintString("\r\n");
		return status;
	}

	return MAX40080_Status_Ok;
}

static int isMax40080Initialized = 0;
static TickType_t lastSampleTime = 0;
static float lastCurrent = 0;
static float lastVoltage = 0;

extern osMutexId_t max40080_sample_accessHandle;

void MAX40080Driver_ThreadEntry() {
	MAX40080_Status mStatus;

	while (1) {
		if (!isMax40080Initialized) {
			mStatus = MAX40080Driver_ReinitializeSensor();
			if (mStatus) {
				// error printed in ReinitializeMAX40080
				vTaskDelay(5);
				continue;
			}

			isMax40080Initialized = 1;
		}

		uint8_t fifoDataCount;
		MAX40080_Interrupt ints;
		mStatus = MAX40080_GetPendingInterruptsAndAvailableFifoDataCount(&ints, &fifoDataCount);
		if (mStatus) {
			UART_PrintString("Error while reading MAX40080 status register. Error code: ");
			UART_PrintFormattedInteger("%d", mStatus);
			UART_PrintString("\r\n");
			isMax40080Initialized = 0;
			continue;
		}

		if (ints & MAX40080_Interrupt_FifoOverflown) {
			UART_PrintString("MAX40080 FIFO overflown.\r\n");

			mStatus = MAX40080_FlushFifo();
			if (mStatus) {
				UART_PrintString("Error while flushing MAX40080 FIFO. Error code: ");
				UART_PrintFormattedInteger("%d", mStatus);
				UART_PrintString("\r\n");
				isMax40080Initialized = 0;
				continue;
			}

			mStatus = MAX40080_ClearPendingInterrupts(MAX40080_Interrupt_FifoOverflown);
			if (mStatus) {
				UART_PrintString("Error while clearing MAX40080 pending interrupt. Error code: ");
				UART_PrintFormattedInteger("%d", mStatus);
				UART_PrintString("\r\n");
				isMax40080Initialized = 0;
				continue;
			}

			vTaskDelay(1);
			continue;
		}

		float current, voltage;

		for (uint8_t i = 0; i < fifoDataCount; i++) {
			mStatus = MAX40080_ReadCurrentAndVoltage(&current, &voltage);
			if (mStatus) {
				if (mStatus != MAX40080_Status_PacketErrorCheckFailed) {
					vTaskDelay(1);
				}

				if (mStatus != MAX40080_Status_PacketErrorCheckFailed && mStatus != MAX40080_Status_FifoIsEmpty) {
					UART_PrintString("Error while reading MAX40080 sample. Error code: ");
					UART_PrintFormattedInteger("%d", mStatus);
					UART_PrintString("\r\n");
					isMax40080Initialized = 0;
				}
				continue;
			}
		}

		osMutexAcquire(max40080_sample_accessHandle, portMAX_DELAY);
		lastCurrent = current;
		lastVoltage = voltage;
		lastSampleTime = xTaskGetTickCount();
		osMutexRelease(max40080_sample_accessHandle);

		vTaskDelay(1);
	}
}

void MAX40080Driver_GetLastSample(TickType_t* timestamp, float* current, float* voltage) {
	osMutexAcquire(max40080_sample_accessHandle, portMAX_DELAY);
	*current = lastCurrent;
	*voltage = lastVoltage;
	*timestamp = lastSampleTime;
	osMutexRelease(max40080_sample_accessHandle);
}
