#include "UiController.h"
#include "stm32l5xx_hal.h"
#include "FreeRTOS.h"
#include "task.h"
#include "ChannelController.h"
#include "ButtonDriver.h"
#include "SevenSegment.h"
#include "AdcDriver.h"

#define LED_BLUE_CH1 GPIO_PIN_1
#define LED_BLUE_CH2 GPIO_PIN_0

#define LETTER_E (DISPA_SEG_A | DISPA_SEG_G | DISPA_SEG_D | DISPA_SEG_F | DISPA_SEG_E)

#define DISPB_SLOT1_LETTER_V (DISPB_SEG_H | DISPB_SEG_G)
#define DISPB_SLOT2_LETTER_V (DISPB_SEG_T | DISPB_SEG_N)
#define DISPB_SLOT1_LETTER_A (DISPB_SEG_H | DISPB_SEG_G  | DISPB_SEG_A | DISPB_SEG_U)
#define DISPB_SLOT2_LETTER_A (DISPB_SEG_B | DISPB_SEG_P  | DISPB_SEG_C | DISPB_SEG_D)

static int selectedChannel = 0;

static void UiController_SelectChannel(int ch) {
	selectedChannel = ch;
	if (ch == 0) {
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH1, 0);
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH2, 1);
	} else if (ch == 1) {
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH1, 1);
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH2, 0);
	} else {
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH1, 1);
		HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH2, 1);
	}
}

static void UiController_StartOrStopChannel(int ch) {
	if (ChannelController_IsRunning(ch)) {
		ChannelController_Stop(ch);
		AdcDriver_SetPotentiometerChangeThreshold(ADC_POTENTIOMETER_THRESHOLD_LOW);
	} else {
		ChannelController_Start(ch);
		AdcDriver_SetPotentiometerChangeThreshold(ADC_POTENTIOMETER_THRESHOLD_HIGH);
	}
}

static void UiController_StartOrStopAllChannels() {
	int isAnyRunning = 0;
	for (int i = 0; i < NUMBER_OF_CHANNELS; i++) {
		if (ChannelController_IsRunning(i)) {
			isAnyRunning = 1;
			break;
		}
	}

	for (int i = 0; i < NUMBER_OF_CHANNELS; i++) {
		if (isAnyRunning) {
			ChannelController_Stop(i);
		} else {
			ChannelController_Start(i);
		}
	}

	if (isAnyRunning) {
		AdcDriver_SetPotentiometerChangeThreshold(ADC_POTENTIOMETER_THRESHOLD_LOW);
	} else {
		AdcDriver_SetPotentiometerChangeThreshold(ADC_POTENTIOMETER_THRESHOLD_HIGH);
	}
}

static char shownUnit = 'V';
static TickType_t lastBlink = 0;
static float lastPotentiometerValue = 3.3;

#define REG_VMIN 1
#define REG_VMAX 10

void UiController_ThreadEntry() {
	HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH1, 0);
	HAL_GPIO_WritePin(GPIOG, LED_BLUE_CH2, 1);

	SevenSegment_Clear();

	/*
	SevenSegment_Clear();
	SevenSegment_SetDigit(DISPA_DIGIT_0_GREEN, DISPA_SEG_A | DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_F | DISPA_SEG_G);
	SevenSegment_SetDigit(DISPA_DIGIT_1_RED, DISPA_SEG_C | DISPA_SEG_B);
	SevenSegment_SetDigit(DISPA_DIGIT_2_RED, DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_F | DISPA_SEG_G);
	SevenSegment_SetDigit(DISPA_DIGIT_2_GREEN, DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_F | DISPA_SEG_G);

	while (1) {
		vTaskDelay(1);
	}
	 */

	while (1) {
		TickType_t now = xTaskGetTickCount();

		if (ButtonDriver_WasPressed(0)) {
			UiController_SelectChannel(0);
		}

		if (ButtonDriver_WasPressed(1)) {
			UiController_SelectChannel(1);
		}

		if (ButtonDriver_WasLongPressed(0)) {
			ChannelController_CalibrateChannel(0);
			continue;
		}

		if (ButtonDriver_WasLongPressed(1)) {
			ChannelController_CalibrateChannel(1);
			continue;
		}

		if (ButtonDriver_WasPressed(2)) {
			UiController_StartOrStopChannel(selectedChannel);
		}

		if (ButtonDriver_WasPressed(3)) {
			UiController_StartOrStopAllChannels();
		}

		float potVal = AdcDriver_GetPotentiometerValue();
		if (potVal != lastPotentiometerValue) {
			lastPotentiometerValue = potVal;
			AdcDriver_SetPotentiometerChangeThreshold(ADC_POTENTIOMETER_THRESHOLD_LOW);
			ChannelController_Stop(selectedChannel);
			ChannelController_SetVoltage(selectedChannel, REG_VMIN + potVal * (REG_VMAX - REG_VMIN));
		}

		if (now - lastBlink > 3000) {
			if (shownUnit == 'V') {
				shownUnit = 'I';
			} else {
				shownUnit = 'V';
			}
			lastBlink = now;
		}

		if (ChannelController_IsError(selectedChannel)) {
			SevenSegment_Clear();
			SevenSegment_SetDigit(DISPA_DIGIT_0_RED, LETTER_E);
		} else if (!ChannelController_IsRunning(selectedChannel)) {
			float v = ChannelController_GetConfiguredVoltage(selectedChannel);
			SevenSegment_PrintFloat(DISPA_DIGIT_0_GREEN, v);
			SevenSegment_PrintFloat(DISPA_DIGIT_0_RED, v);
			SevenSegment_SetDigit(DISPB_CYC1, DISPB_SLOT1_LETTER_V);
			SevenSegment_SetDigit(DISPB_CYC2, DISPB_SLOT2_LETTER_V);
		} else {
			if (shownUnit == 'I') {
				float current = ChannelController_GetActualCurrent(selectedChannel);
				SevenSegment_ClearA_Green();
				SevenSegment_PrintFloat(DISPA_DIGIT_0_RED, current);
				SevenSegment_SetDigit(DISPB_CYC1, DISPB_SLOT1_LETTER_A);
				SevenSegment_SetDigit(DISPB_CYC2, DISPB_SLOT2_LETTER_A);
			} else {
				float voltage = ChannelController_GetActualVoltage(selectedChannel);
				SevenSegment_ClearA_Red();
				SevenSegment_PrintFloat(DISPA_DIGIT_0_GREEN, voltage);
				SevenSegment_SetDigit(DISPB_CYC1, DISPB_SLOT1_LETTER_V);
				SevenSegment_SetDigit(DISPB_CYC2, DISPB_SLOT2_LETTER_V);
			}
		}

		vTaskDelay(1);
	}
}
